@@ -1,9 +1,7 @@
 def get_classes():
     
     from Navigator import Navigator
-    from Preferences import Preferences
     from GenericThumbnailer import GenericThumbnailer
     return [Navigator,
-            #Preferences,
             GenericThumbnailer]
 
